/*
 * Copyright (c) 2017 SpaceToad and the BuildCraft team
 * This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy of the MPL was not
 * distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/
 */

package buildcraft.lib.client.render.fluid;

import java.io.IOException;
import java.util.Arrays;
import java.util.function.Function;

import net.minecraft.client.Minecraft;
import net.minecraft.client.renderer.texture.PngSizeInfo;
import net.minecraft.client.renderer.texture.TextureAtlasSprite;
import net.minecraft.client.resources.IResource;
import net.minecraft.client.resources.IResourceManager;
import net.minecraft.util.ResourceLocation;

import buildcraft.api.core.BCLog;

import buildcraft.lib.misc.SpriteUtil;

public class SpriteFluidFrozen extends TextureAtlasSprite {
    /** The source sprite of this fluid. */
    public final ResourceLocation srcLocation;
    private int[][] data = null;

    public SpriteFluidFrozen(ResourceLocation srcLocation) {
        super("buildcraftlib:fluid_" + srcLocation.toString().replace(':', '_') + "_convert_frozen");
        this.srcLocation = srcLocation;
    }

    @Override
    public boolean hasCustomLoader(IResourceManager manager, ResourceLocation location) {
        return true;
    }

    @Override
    public boolean load(IResourceManager manager, ResourceLocation location, Function<ResourceLocation, TextureAtlasSprite> textureGetter) {
        location = SpriteUtil.transformLocation(srcLocation);
        TextureAtlasSprite src = Minecraft.func_71410_x().func_147117_R().getTextureExtry(srcLocation.toString());
        if (src == null) {
            BCLog.logger.warn("[lib.fluid] Failed to create a frozen sprite of " + srcLocation.toString() + " as the source sprite wasn't able to be loaded!");
            return true;
        }

        if (src.func_110970_k() <= 0) {
            if (src.hasCustomLoader(manager, location)) {
                src.load(manager, location, textureGetter);
            } else {
                try {
                    PngSizeInfo pngsizeinfo = PngSizeInfo.func_188532_a(manager.func_110536_a(location));
                    try (IResource resource = manager.func_110536_a(location)) {
                        src.func_188538_a(pngsizeinfo, resource.func_110526_a("animation") != null);
                        src.func_188539_a(resource, Minecraft.func_71410_x().field_71474_y.field_151442_I + 1);
                    }
                } catch (IOException io) {
                    io.printStackTrace();
                }
            }
        }

        if (src.func_110970_k() > 0) {
            int widthOld = src.func_94211_a();
            int heightOld = src.func_94216_b();
            field_130223_c = widthOld * 2;
            field_130224_d = heightOld * 2;

            int[][] srcData = src.func_147965_a(0);

            data = new int[Minecraft.func_71410_x().field_71474_y.field_151442_I + 1][];
            for (int m = 0; m < data.length; m++) {
                data[m] = new int[field_130223_c * field_130224_d / (m + 1) / (m + 1)];
            }
            int[] relData = srcData[0];
            if (relData.length < (field_130223_c * field_130224_d / 4)) {
                Arrays.fill(data[0], 0xFF_FF_FF_00);
            } else {
                for (int x = 0; x < field_130223_c; x++) {
                    int fx = (x % widthOld) * heightOld;
                    for (int y = 0; y < field_130224_d; y++) {
                        int fy = y % heightOld;
                        data[0][x * field_130224_d + y] = relData[fx + fy];
                    }
                }
            }
        } else {
            // Urm... idk
            BCLog.logger.warn("[lib.fluid] Failed to create a frozen sprite of " + src.func_94215_i() + " as the source sprite didn't have any frames!");
            return true;
        }
        return false;
    }

    @Override
    public int func_110970_k() {
        return data == null ? 0 : 1;
    }

    @Override
    public int[][] func_147965_a(int index) {
        return data;
    }

    @Override
    public float func_94214_a(double u) {
        return super.func_94214_a(u / 2 + 4);
    }

    @Override
    public float func_94207_b(double v) {
        return super.func_94207_b(v / 2 + 4);
    }
}
